-- Specialisation which sets the visibility of hydraulics
-- @ author: rafftnix
-- @ date: 29.07.2013

SetHydraulicsVisibility = {};

function SetHydraulicsVisibility.prerequisitesPresent(specializations)
    return true;
end;

function SetHydraulicsVisibility:load(xmlFile)
	self.setHydState = SpecializationUtil.callSpecializationsFunction("setHydState");
	self.getIsHydTaken = SetHydraulicsVisibility.getIsHydTaken;

	self.frontHydActive = true;
	self.rearHydActive = true;
	
	self.frontAttacherDescIndex = getXMLInt(xmlFile, "vehicle.attachableHydraulics.frontHydraulic#attacherJointDescIndex");
	self.rearAttacherDescIndex = getXMLInt(xmlFile, "vehicle.attachableHydraulics.rearHydraulic#attacherJointDescIndex");
	
	self.frontHydDesc = self.attacherJoints[self.frontAttacherDescIndex];
	self.rearHydDesc = self.attacherJoints[self.rearAttacherDescIndex];
	
	self.frontHydDesc.oldPos = { getTranslation(self.frontHydDesc.jointTransform) }
	self.rearHydDesc.oldPos = { getTranslation(self.rearHydDesc.jointTransform) }
	
	self.frontHyd = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.attachableHydraulics.frontHydraulic#index"));
	self.rearHyd = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.attachableHydraulics.rearHydraulic#index"));
	
	self.showFillWarnTime = 0;
	self.showHydraulicWarnTime = 0;
	
	self.tBD = Utils.indexToObject(self.components, "0>6|9|0|0|0");
	self.tSDL = Utils.indexToObject(self.components, "0>6|9|0|0|1");
	self.tSDR = Utils.indexToObject(self.components, "0>6|9|0|0|2");
	self.tBHL = Utils.indexToObject(self.components, "0>6|9|0|0|0|0");
	self.tBHR = Utils.indexToObject(self.components, "0>6|9|0|0|0|1");
	self.tP = Utils.indexToObject(self.components, "0>6|9|0|0|5");
end;

function SetHydraulicsVisibility:readStream(streamId, connection)
	local frontHydActive = streamReadBool(streamId);
	local rearHydActive = streamReadBool(streamId);
	self:setHydState(frontHydActive, rearHydActive, true);
end;

function SetHydraulicsVisibility:writeStream(streamId, connection)
	streamWriteBool(streamId, self.frontHydActive);
	streamWriteBool(streamId, self.rearHydActive);
end;

function SetHydraulicsVisibility:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then
		local frontHydActive = getXMLBool(xmlFile, key.."#frontHydActive");
		local rearHydActive = getXMLBool(xmlFile, key.."#rearHydActive");
		self:setHydState(frontHydActive, rearHydActive, true);
	end;

	return BaseMission.VEHICLE_LOAD_OK;
end;

function SetHydraulicsVisibility:getSaveAttributesAndNodes(nodeIdent)	
	local attributes = 'frontHydActive="' .. tostring(self.frontHydActive) .. '" ';
	attributes = attributes .. 'rearHydActive="' .. tostring(self.rearHydActive) .. '"';
	return attributes, "";
end;

function SetHydraulicsVisibility:delete()
  
end;

function SetHydraulicsVisibility:mouseEvent(posX, posY, isDown, isUp, button)
end;

function SetHydraulicsVisibility:keyEvent(unicode, sym, modifier, isDown)
end;

function SetHydraulicsVisibility:update(dt)
	if self:getIsActiveForInput(true) then
		if InputBinding.hasEvent(InputBinding.FRONTHYDRAULIC) and table.getn(self.attachedImplements) == 0 then
			self:setHydState(not self.frontHydActive, self.rearHydActive);
		end;
		if InputBinding.hasEvent(InputBinding.REARHYDRAULIC) and self.fillLevel == 0 and table.getn(self.attachedImplements) == 0 then
			self:setHydState(self.frontHydActive, not self.rearHydActive);
		end;
		if InputBinding.hasEvent(InputBinding.REARHYDRAULIC) and self.fillLevel > 0 then
            self.showFillWarnTime = self.time + 3000;
        end;
        
        if InputBinding.hasEvent(InputBinding.REARHYDRAULIC) and table.getn(self.attachedImplements) > 0 then
            self.showHydraulicWarnTime = self.time + 3000;
        end;
        
        if InputBinding.hasEvent(InputBinding.FRONTHYDRAULIC) and table.getn(self.attachedImplements) > 0 then
            self.showHydraulicWarnTime = self.time + 3000;
        end;
	end;
end;

function SetHydraulicsVisibility:draw()
	--if table.getn(self.attachedImplements) == 0 then
	if self:getIsActiveForInput(true) then
		if self.frontHydActive then
			g_currentMission:addHelpButtonText(g_i18n:getText("detachFrontHydraulic"), InputBinding.FRONTHYDRAULIC);
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("attachFrontHydraulic"), InputBinding.FRONTHYDRAULIC);
		end;
		
		if self.rearHydActive then
			g_currentMission:addHelpButtonText(g_i18n:getText("detachRearHydraulic"), InputBinding.REARHYDRAULIC);
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("attachRearHydraulic"), InputBinding.REARHYDRAULIC);
		end;
	--else
	--	g_currentMission:addExtraPrintText(g_i18n:getText("detachAllVehicles"));
	end;
	
	if self.showFillWarnTime > self.time then
		g_currentMission:addWarning(g_i18n:getText("warning_fillLevel"), 0, 0.03);
	end;
	
	if self.showHydraulicWarnTime > self.time then
		g_currentMission:addWarning(g_i18n:getText("detachAllVehicles"), 0, 0.03);
	end;
end;

function SetHydraulicsVisibility:setHydState(frontHydActive, rearHydActive, noEventSend)
	setVisibility(self.frontHyd, frontHydActive);
	setVisibility(self.rearHyd, rearHydActive);
	
	if self.frontHydActive ~= frontHydActive then
		if frontHydActive then
			setTranslation(self.frontHydDesc.jointTransform, unpack(self.frontHydDesc.oldPos));
		else
			setTranslation(self.frontHydDesc.jointTransform, 20000, 20000, 20000);
		end;
	end;
	
	if self.rearHydActive ~= rearHydActive then
		if rearHydActive then
			setTranslation(self.rearHydDesc.jointTransform, unpack(self.rearHydDesc.oldPos));
			
			setVisibility(self.tBD, false);
			setVisibility(self.tSDL, false);
			setVisibility(self.tSDR, false);
			setVisibility(self.tBHL, false);
			setVisibility(self.tBHR, false);
			setVisibility(self.tP, false);
			self.allowFillFromAir = false;
		else
			setTranslation(self.rearHydDesc.jointTransform, 20000, 20000, 20000);
			
			setVisibility(self.tBD, true);
			setVisibility(self.tSDL, true);
			setVisibility(self.tSDR, true);
			setVisibility(self.tBHL, true);
			setVisibility(self.tBHR, true);
			setVisibility(self.tP, true);
			self.allowFillFromAir = true;
		end;
	end;
	
	self.frontHydActive = frontHydActive;
	self.rearHydActive = rearHydActive;

	if not noEventSend then
		SetHydraulicsVisibilityEvent.sendEvent(self, frontHydActive, rearHydActive);
	end;
end;

SetHydraulicsVisibilityEvent = {};
SetHydraulicsVisibilityEvent_mt = Class(SetHydraulicsVisibilityEvent, Event);
InitEventClass(SetHydraulicsVisibilityEvent, "SetHydraulicsVisibilityEvent");
function SetHydraulicsVisibilityEvent:emptyNew()
    local self = Event:new(SetHydraulicsVisibilityEvent_mt);
    self.className="SetHydraulicsVisibilityEvent";
    return self;
end;

function SetHydraulicsVisibilityEvent:new(object, frontHydActive, rearHydActive)
    local self = SetHydraulicsVisibilityEvent:emptyNew(); 
    self.object = object; 
	self.frontHydActive = frontHydActive;
	self.rearHydActive = rearHydActive;
    return self;
end;

function SetHydraulicsVisibilityEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.object));
    streamWriteBool(streamId, self.frontHydActive);
	streamWriteBool(streamId, self.rearHydActive);
end;

function SetHydraulicsVisibilityEvent:readStream(streamId, connection)
    self.object = networkGetObject(streamReadInt32(streamId));
    self.frontHydActive = streamReadBool(streamId);
	self.rearHydActive = streamReadBool(streamId);
    self:run(connection);
end;

function SetHydraulicsVisibilityEvent:run(connection)
	self.object:setHydState(self.frontHydActive, self.rearHydActive, true);
	if not connection:getIsServer() then
		g_server:broadcastEvent(SetHydraulicsVisibilityEvent:new(self.object, self.frontHydActive, self.rearHydActive), nil, connection, self.object);
	end; 
end;

function SetHydraulicsVisibilityEvent.sendEvent(object, frontHydActive, rearHydActive)
	if g_server ~= nil then   
		g_server:broadcastEvent(SetHydraulicsVisibilityEvent:new(object, frontHydActive, rearHydActive), nil, nil, object); 
	else  
		g_client:getServerConnection():sendEvent(SetHydraulicsVisibilityEvent:new(object, frontHydActive, rearHydActive)); 
	end;
end;