--
-- Specialisation for MB Trac 1600 Turbo
-- 
-- Author: Mofi, Face, John Deere 6930


UnimogU1600 = {};

function UnimogU1600.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function UnimogU1600:load(xmlFile)
    
	local gaspedalNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.gaspedal#index"));
    if gaspedalNode ~= nil then
        self.gaspedal = {};
        self.gaspedal.node = gaspedalNode;    
		self.gaspedal.maxSpeed = getXMLInt(xmlFile, "vehicle.gaspedal#maxSpeed");
		local x1,y1,z1 = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.gaspedal#minRot"));
		local x2,y2,z2 = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.gaspedal#maxRot"));		
		self.gaspedal.rotMin = {math.rad(Utils.getNoNil(x1,0)),math.rad(Utils.getNoNil(y1,0)),math.rad(Utils.getNoNil(z1,0))};
		self.gaspedal.rotMax = {math.rad(Utils.getNoNil(x2,0)),math.rad(Utils.getNoNil(y2,0)),math.rad(Utils.getNoNil(z2,0))};
    end;
	
    local bremspedalNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.bremspedal#index"));
    if bremspedalNode ~= nil then
        self.bremspedal = {};
        self.bremspedal.node = bremspedalNode;
		local x1,y1,z1 = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.gaspedal#minRot"));
		local x2,y2,z2 = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.gaspedal#maxRot"));
		self.bremspedal.rotMin = {math.rad(Utils.getNoNil(x1,0)),math.rad(Utils.getNoNil(y1,0)),math.rad(Utils.getNoNil(z1,0))};			
		self.bremspedal.rotMax = {math.rad(Utils.getNoNil(x2,0)),math.rad(Utils.getNoNil(y2,0)),math.rad(Utils.getNoNil(z2,0))};
        self.bremspedal.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.bremspedal#rotTime"), 1)*1000;
        self.bremspedalIsActive = false;
    end;	
	
    local blinkerhebelNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.blinkerhebel#index"));
    if blinkerhebelNode ~= nil then
        self.blinkerhebel = {};
        self.blinkerhebel.node = blinkerhebelNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.blinkerhebel#minRot"));
        self.blinkerhebel.minRot = {};
        self.blinkerhebel.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.blinkerhebel.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.blinkerhebel.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.blinkerhebel#rotLeft"));
        self.blinkerhebel.leftRot = {};
        self.blinkerhebel.leftRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.blinkerhebel.leftRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.blinkerhebel.leftRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.blinkerhebel#rotRight"));
        self.blinkerhebel.rightRot = {};
        self.blinkerhebel.rightRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.blinkerhebel.rightRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.blinkerhebel.rightRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.blinkerhebel.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.blinkerhebel#rotTime"), 1)/1000;
		self.blinkerhebel.lastDir = 1;
    end;
	
	self.drehzahlnadel = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.needles.rpmNeedle#index"));
	self.tachonadel = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.needles.speedNeedle#index"));
	self.tanknadel = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.needles.fuelNeedle#index"));
	self.needles = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.needles.needles#index"));
	self.drehzahlnadelLight = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.needles.rpmNeedle#light"));
	self.tachonadelLight = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.needles.speedNeedle#light"));
	self.tanknadelLight = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.needles.fuelNeedle#light"));
	self.needlesLight = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.needles.needles#light"));
	self.tankicon = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.tankicon#index"));
	self.tipicon = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.tipicon#index"));
	self.hazardSwitch = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.cockpitSwitches#hazardSwitch"));
	self.lightSwitch = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.cockpitSwitches#lightSwitch"));
	self.beaconLightSwitch = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.cockpitSwitches#beaconLightSwitch"));
	self.seat = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.seat#index"));
	self.cockpitLights = Utils.indexToObject(self.components, "0>6|1|3|0");
	
	self.drivingShaftLeft = {};
	self.drivingShaftLeft.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.drivingShaft#left"));
	self.drivingShaftLeft.speedFactor = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.drivingShaft#speedFactor"), 1);
	self.drivingShaftRight = {};
	self.drivingShaftRight.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.drivingShaft#right"));
	
	self.tipAnimtionBackup = self.tipAnimations[3];
    self.tipReferencePointBackup = self.tipReferencePoints[3];
    
    self.frontTopArm = Utils.indexToObject(self.components, "0>6|0|9|2");
    self.backTopArm = Utils.indexToObject(self.components, "0>6|0|10|2"); 
    self.tAN = Utils.indexToObject(self.components, "0>6|0|8");
    
    self.allowFillFromAir = false;
    self.allowTipDischarge = false;       
end;

function UnimogU1600:delete()
end;

function UnimogU1600:readStream(streamId, connection)
end;

function UnimogU1600:writeStream(streamId, connection)
end;

function UnimogU1600:mouseEvent(posX, posY, isDown, isUp, button)
end;

function UnimogU1600:keyEvent(unicode, sym, modifier, isDown)
end;

function UnimogU1600:update(dt)
    if self:getIsActive() then        
        local sx,sy,sz = getTranslation(self.seat);
        local _,ry,_ = getTranslation(self.wheels[1].repr);
        setTranslation(self.seat, sx, (ry+4.893) /3, sz);
        
        local wx,wy,wz = getRotation(self.wheels[1].driveNode);
		setRotation(self.drivingShaftLeft.node, wx*self.drivingShaftLeft.speedFactor ,0,0 );
        setRotation(self.drivingShaftRight.node, wx*self.drivingShaftLeft.speedFactor ,0,0 );   		
	end;
    
    if self.rLights == nil then
        for i=1, table.getn(self.interactiveObjects) do
            table.remove(self.interactiveObjects, 9);
            break;
        end;
    end;	
end;

function UnimogU1600:updateTick(dt)

    if self:getIsActive() then	
		
		if self.lightsTypesMask == 1 or self.lightsTypesMask == 3 then
	        setVisibility(self.cockpitLights, true);
	    else
	        setVisibility(self.cockpitLights, false);
        end;
		
		-- check if currentcam is indoorcam
		if self.camIndex == 2 then
		    
		    if self.rLights ~= nil then
                if self.rLights.blw.a then
	               setRotation(self.hazardSwitch, Utils.degToRad(-14), 0, 0);
	            else
	               setRotation(self.hazardSwitch, Utils.degToRad(0), 0, 0);
	           end;
	        end;
	        
	        if self.beaconLightsActive then
	            setRotation(self.beaconLightSwitch, Utils.degToRad(-14), 0, 0);
	        else
	            setRotation(self.beaconLightSwitch, Utils.degToRad(0), 0, 0);
	        end;

	        if (self.lightsTypesMask == 1) or (self.lightsTypesMask == 3) then
	            setRotation(self.lightSwitch, 0, 0, Utils.degToRad(80));
	        else
	            setRotation(self.lightSwitch, 0, 0, Utils.degToRad(0));
            end;
		
			if self.gaspedal ~= nil then		
				local percent = (self.lastSpeed*self.speedDisplayScale*3600) / self.gaspedal.maxSpeed;
				local minRot = self.gaspedal.rotMin;
				local maxRot = self.gaspedal.rotMax;
					
				local x = minRot[1] + (maxRot[1] - minRot[1]) * percent;
				local y = minRot[2] + (maxRot[2] - minRot[2]) * percent;
				local z = minRot[3] + (maxRot[3] - minRot[3]) * percent;
				setRotation(self.gaspedal.node, x, y, z);
			end;
			
			-- check for braking
			if self.bremspedal ~= nil then
				local x, y, z = getRotation(self.bremspedal.node);
				local rot = {x,y,z};
				local direction = 1;
				local braking = math.abs(Utils.sign(self.lastAcceleration)+ (-self.movingDirection * direction)) == 2;
				local newRot = Utils.getMovedLimitedValues(rot, self.bremspedal.rotMax, self.bremspedal.rotMin, 3, self.bremspedal.rotTime, dt, braking);
				setRotation(self.bremspedal.node, unpack(newRot));
			end;
			
			if self.blinkerhebel ~= nil and self.rLights ~= nil then
				local dir = 1;
				local hasRotChanged = false;
				local destRot = self.blinkerhebel.minRot;
				if not (self.rLights.blr.a and self.rLights.bll.a) then
					if self.rLights.bll.a then
						destRot = self.blinkerhebel.leftRot;
						self.blinkerhebel.lastDir = 1;		
					elseif self.rLights.blr.a then
						destRot = self.blinkerhebel.rightRot;
						self.blinkerhebel.lastDir = -1;
						dir = -1;
					else
						if not self.rLights.blr.a and not self.rLights.bll.a then
							if self.blinkerhebel.lastDir ~= -1 then
								dir = -1;
								if self.blinkerhebel.lastDir ~= 1 then
								    dir = 1;
								end;
							end;
						end;
					end;
				else
					local x,y,z = getRotation(self.blinkerhebel.node);
					if y > 0 then
						dir = -1;
					end;
				end;
				
				local rot = {getRotation(self.blinkerhebel.node)};
				for i=1, 3 do
					
					local newRot = AnimatedVehicle.getMovedLimitedValue(rot[i], destRot[i], self.blinkerhebel.rotTime*dir, dt);
					if rot[i] ~= newRot then
						hasRotChanged = true;
						rot[i] = newRot;
					end;
				end;
				if hasRotChanged then
					setRotation(self.blinkerhebel.node, unpack(rot));
				end;
			end;			
		else
			setRotation(self.gaspedal.node, unpack(self.gaspedal.rotMin));
			setRotation(self.bremspedal.node, unpack(self.bremspedal.rotMin));
		end;
		
		if self.isMotorStarted then
			local kmh = math.min(100, math.max(0, self.lastSpeed*self.speedDisplayScale*3600));
			local rotateDrehzahlnadel = (((kmh / 30)*100) -20);
			local rotateTachonadel = (((kmh / 50)*100) -15);
			local currentFuelPercentage = 0;
			local fuelWarnPercentage = 20;
			if self.fuelCapacity > 0 then
				currentFuelPercentage = (self.fuelFillLevel / self.fuelCapacity + 0.0001) * 100;
			end;
			if currentFuelPercentage < fuelWarnPercentage then
				setVisibility(self.tankicon, true);
			else
				setVisibility(self.tankicon, false);
			end;
			local rotateTanknadel = ((currentFuelPercentage / 111) * 100);
			if (self.lightsTypesMask == 1) or (self.lightsTypesMask == 3) then
				setVisibility(self.drehzahlnadel, false);
				setVisibility(self.tachonadel, false);
				setVisibility(self.tanknadel, false);
				setVisibility(self.needles, false);
				setVisibility(self.drehzahlnadelLight, true);
				setVisibility(self.tachonadelLight, true);
				setVisibility(self.tanknadelLight, true);
                setVisibility(self.needlesLight, true);    
				setRotation(self.drehzahlnadelLight, 0, Utils.degToRad( - rotateDrehzahlnadel), 0);
				setRotation(self.tachonadelLight, 0, Utils.degToRad( - rotateTachonadel), 0);
				setRotation(self.tanknadelLight, 0, Utils.degToRad(rotateTanknadel), 0);
			else
				setVisibility(self.drehzahlnadel, true);
				setVisibility(self.tachonadel, true);
				setVisibility(self.tanknadel, true);
				setVisibility(self.needles, true);
				setVisibility(self.drehzahlnadelLight, false);
				setVisibility(self.tachonadelLight, false);
				setVisibility(self.tanknadelLight, false);
				setVisibility(self.needlesLight, false);
				setRotation(self.drehzahlnadel, 0, Utils.degToRad( - rotateDrehzahlnadel), 0);
				setRotation(self.tachonadel, 0, Utils.degToRad( - rotateTachonadel), 0);
				setRotation(self.tanknadel, 0, Utils.degToRad(rotateTanknadel), 0);
			end;
		else
			setRotation(self.drehzahlnadel, 0, 0, 0);
			setRotation(self.tachonadel, 0, 0, 0);
			setRotation(self.tanknadel, 0, 0, 0);
			setRotation(self.drehzahlnadelLight, 0, 0, 0);
			setRotation(self.tachonadelLight, 0, 0, 0);
			setRotation(self.tanknadelLight, 0, 0, 0);
			setVisibility(self.tankicon, false);
		end;
	end;
	
	if (self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN or self.tipState == Trailer.TIPSTATE_CLOSING) then
	    setVisibility(self.tipicon, true);
	else
		setVisibility(self.tipicon, false);
	end;
	
	if self.rearHydActive then
        self.bodyLimits.allowChange = false;
        if self.currentBodyType == 2 then 
            self:setBodyTypeId(self.currentBodyType - 1);
        end;
        if self.toggleTipSideCount ~= nil then
            self.toggleTipSideActive = false;
        end;
        self.allowTipDischarge = false;
    else
        self.bodyLimits.allowChange = true;
        local vTAN = getVisibility(self.tAN);
        if vTAN == false then
            setVisibility(self.tAN, true);
        end;
        if self.toggleTipSideCount ~= nil then
            self.toggleTipSideActive = true;
        end;
        self.allowTipDischarge = true;
    end;
    
    if self.currentBodyType == 2 and self.toggleTipSideCount ~= nil then
        self.toggleTipSideCount = 2;
    end;
    
    if self.currentBodyType == 1 and self.toggleTipSideCount ~= nil and table.getn(self.tipAnimations) == 3 then
        self.toggleTipSideCount = 3;
    end;
             
end;

function UnimogU1600:draw()
end;

function UnimogU1600:attachImplement(implement)
	
	local jointType = implement.object.attacherJoint.jointType;
	local jointIndex = implement.jointDescIndex;
	
	if jointType == Vehicle.JOINTTYPE_TRAILER then
	   if jointIndex == 6 then
            for i=1, table.getn(self.tipAnimations) do
                table.remove(self.tipAnimations, 3);
                break;
            end;
            for i=1, table.getn(self.tipReferencePoints) do
                table.remove(self.tipReferencePoints, 3);
                break;
            end;
            if self.toggleTipSideCount ~= nil then
                self.toggleTipSideCount = 2;
            end;
        end;
    elseif jointType == Vehicle.JOINTTYPE_TRAILERLOW then
	   if jointIndex == 5 then
            for i=1, table.getn(self.tipAnimations) do
                table.remove(self.tipAnimations, 3);
                break;
            end;
            for i=1, table.getn(self.tipReferencePoints) do
                table.remove(self.tipReferencePoints, 3);
                break;
            end;
            if self.toggleTipSideCount ~= nil then
                self.toggleTipSideCount = 2;
            end;
        end;
    elseif jointType == Vehicle.JOINTTYPE_IMPLEMENT then
        if jointIndex == 1 then
            setVisibility(self.frontTopArm, true);
        end;
        if jointIndex == 2 then
            setVisibility(self.backTopArm, true);
            setVisibility(self.tAN, false);
        end;
    end;
end;

function UnimogU1600:detachImplement(implementIndex)

    local implement = self.attachedImplements[implementIndex];
	local jointIndex = implement.jointDescIndex;
	
	if jointIndex == 6 then
        for i=1, table.getn(self.tipAnimations) do
            local tipAnimationBackup = self.tipAnimtionBackup;
            table.insert(self.tipAnimations, tipAnimationBackup);
        end;
        for i=1, table.getn(self.tipReferencePoints) do
            local tipReferencePointBackup = self.tipReferencePointBackup;
            table.insert(self.tipReferencePoints, tipReferencePointBackup);
        end;
        if self.toggleTipSideCount ~= nil then
            self.toggleTipSideCount = 3;
        end;
    elseif jointIndex == 5 then
        for i=1, table.getn(self.tipAnimations) do
            local tipAnimationBackup = self.tipAnimtionBackup;
            table.insert(self.tipAnimations, tipAnimationBackup);
        end;
        for i=1, table.getn(self.tipReferencePoints) do
            local tipReferencePointBackup = self.tipReferencePointBackup;
            table.insert(self.tipReferencePoints, tipReferencePointBackup);
        end;
        if self.toggleTipSideCount ~= nil then
            self.toggleTipSideCount = 3;
        end;
    elseif jointIndex == 1 then
        setVisibility(self.frontTopArm, false);
    elseif jointIndex == 2 then
        setVisibility(self.backTopArm, false);
        setVisibility(self.tAN, true);
    end;
end;

function UnimogU1600:onLeave()  
end;

function UnimogU1600:onEnter()
end;

